//@ts-check
"use strict";

/**
@type {{
  pixiRoot: PIXI.Container;
  loop: PIXI.AnimatedSprite;
  title: PIXI.Text;
  endState: EndState;

  loopTimer: number;
  loopForward: boolean;
}}
*/
let win = null;

/**
@typedef EndState
@type {{
  savedFolks: number,
}}
 */

function registerWinResources() {
  app.loader.add([
    "win/loop.png",
    "win/loop.sprite.json",
  ]);
}

/** @param {EndState} endState */
function enterWin(endState) {
  ensureMenuMusic("success");

  const pixiRoot = new PIXI.Container();
  app.stage.addChildAt(pixiRoot, 0);

  const loopSpritesheet = getSpritesheet("win/loop");
  const loop = new PIXI.AnimatedSprite(loopSpritesheet.animations["loop"]);
  loop.anchor.x = 0.5;
  loop.play();
  pixiRoot.addChild(loop);

  const title = new PIXI.Text(getText("win.title"), new PIXI.TextStyle({
    fontFamily: "Arvo", fontSize: 192, fontWeight: "bold", align: "center",
    fill: "#fff", strokeThickness: 16, stroke: "#513a43", lineJoin: "bevel"
  }));
  title.anchor.set(0.5);
  title.scale.set(0.25);
  title.position.set(gameWidth / 2, gameHeight * 3 / 8);
  pixiRoot.addChild(title);

  let textID = endState.savedFolks > 0 ? "win.textFriends" : "win.textAlone";
  const text = new PIXI.Text(getText(textID), new PIXI.TextStyle({
    fontFamily: "Arvo", fontSize: 128, align: "center",
    fill: "#ffddb2", strokeThickness: 16, stroke: "#513a43", lineJoin: "bevel"
  }));
  text.anchor.set(0.5);
  text.scale.set(0.25);
  text.position.set(gameWidth / 2, gameHeight * 5 / 8);
  pixiRoot.addChild(text);

  win = {
    pixiRoot,
    loop,
    title,
    endState,
    loopTimer: 0,
    loopForward: true,
  };

  app.ticker.add(animateWin);
  document.addEventListener("keydown", onWinKeyDown);
}

function leaveWin(enter) {
  app.ticker.remove(animateWin);
  document.removeEventListener("keydown", onWinKeyDown);
  app.stage.children[0].destroy();
  win = null;

  enter();
}

function animateWin() {
  win.pixiRoot.scale.set(app.renderer.height / gameHeight);

  const dt = PIXI.Ticker.shared.elapsedMS / 1000;

  win.loopTimer += 0.1 * (win.loopForward ? dt : -dt);
  if (Math.abs(win.loopTimer) > 1) {
    win.loopTimer = Math.sign(win.loopTimer);
    win.loopForward = !win.loopForward;
  }

  // win.loop.x = gameWidth / 2 + Math.sign(win.loopTimer) * easeOutQuad(Math.abs(win.loopTimer)) * 80;
  win.loop.x = gameWidth / 2;
}

function onWinKeyDown(event) {
  leaveWin(enterMainMenu);
  openCredits();
}